package com.itmill.toolkit.demo;

import java.io.IOException;
import java.io.Serializable;
import java.util.Iterator;

import com.itmill.toolkit.Application;
import com.itmill.toolkit.service.ApplicationContext.TransactionListener;
import com.itmill.toolkit.ui.Button;
import com.itmill.toolkit.ui.GridLayout;
import com.itmill.toolkit.ui.Label;
import com.itmill.toolkit.ui.Window;

public class Calc extends com.itmill.toolkit.Application implements
        Serializable, TransactionListener {

    private static final long serialVersionUID = -2185106514237814051L;

    private double current = 0.0;
    private double stored = 0.0;
    private char lastOperationRequested = 'C';

    // User interface components
    private final Label display = new Label("0.0");
    private final GridLayout layout = new GridLayout(4, 5);

    // Application initialization creates UI and connects it to business logic
    @Override
    public void init() {

        // Place the layout to the browser main window
        setMainWindow(new Window("Calculator Application", layout));

        // Create and add the components to the layout
        layout.addComponent(display, 0, 0, 3, 0);
        for (String caption : new String[] { "7", "8", "9", "/", "4", "5", "6",
                "*", "1", "2", "3", "-", "0", "=", "C", "+" }) {
            Button button = new Button(caption, new Button.ClickListener() {
                public void buttonClick(Button.ClickEvent event) {

                    // On button click, calculate and show the result
                    display.setValue(calculate(event.getButton()));
                }
            });
            layout.addComponent(button);
        }

        getContext().addTransactionListener(this);
    }

    // Calculator "business logic" implemented here to keep the example minimal
    private double calculate(Button buttonClicked) {
        char requestedOperation = buttonClicked.getCaption().charAt(0);
        if ('0' <= requestedOperation && requestedOperation <= '9') {
            current = current * 10
                    + Double.parseDouble("" + requestedOperation);
            return current;
        }
        switch (lastOperationRequested) {
        case '+':
            stored += current;
            break;
        case '-':
            stored -= current;
            break;
        case '/':
            stored /= current;
            break;
        case '*':
            stored *= current;
            break;
        case 'C':
            stored = current;
            break;
        }
        lastOperationRequested = requestedOperation;
        current = 0.0;
        if (requestedOperation == 'C') {
            stored = 0.0;
        }
        return stored;
    }

    private int transactionCount = 0;
    private final long id = System.currentTimeMillis();

    public void transactionEnd(Application application, Object transactionData) {
        System.err.println("Calc: End transaction " + transactionCount
                + ": id=" + id + " current=" + current + " display=" + display);
        System.err.print("Apps: ");
        for (Iterator<Application> i = getContext().getApplications()
                .iterator(); i.hasNext();) {
            Application a = i.next();
            System.err.print(a.getClass());
            if (a instanceof Calc) {
                System.err.print("(id=" + id + " display=" + ((Calc) a).display
                        + ")");
            }
            if (i.hasNext()) {
                System.err.print(",");
            }
        }
        System.err.println("");
    }

    public void transactionStart(Application application, Object transactionData) {
        transactionCount++;
        System.err.println("Calc: Start transaction " + transactionCount
                + ": id=" + id + " current=" + current + " display=" + display);
    }

    /* Better logging for serialization problems */
    private void writeObject(java.io.ObjectOutputStream out) throws IOException {
        System.err.println("Starting to serialize Calc id=" + id + " current="
                + current + " display=" + display);
        out.defaultWriteObject();
        System.err.println("Done serializing Calc id=" + id + " current="
                + current + " display=" + display);
    }

    /* Special serialization to handle method references */
    private void readObject(java.io.ObjectInputStream in) throws IOException,
            ClassNotFoundException {
        System.err.println("Starting to deserialize Calc id=" + id
                + " current=" + current + " display=" + display);
        in.defaultReadObject();
        System.err.println("Done deserializing Calc id=" + id + " current="
                + current + " display=" + display);
    }

}
